@@ -39,7 +39,7 @@ use SNMP::Info;
 
 use vars qw/$VERSION %FUNCS %GLOBALS %MIBS %MUNGE/;
 
-$VERSION = '2.06';
+$VERSION = '3.11';
 
 %MIBS = (
     %SNMP::Info::MIBS,
@@ -83,7 +83,8 @@ $VERSION = '2.06';
     'airespace_ess_qos'       => 'bsnDot11EssQualityOfService',
     'airespace_ess_ifname'    => 'bsnDot11EssInterfaceName',
     'airespace_ess_aclname'   => 'bsnDot11EssAclName',
-
+    'airespace_ess_bcast'     => 'bsnDot11EssBroadcastSsid',
+    
     # AIRESPACE-WIRELESS-MIB::bsnAPTable
     'airespace_ap_mac'      => 'bsnAPDot3MacAddress',
     'airespace_ap_name'     => 'bsnAPName',
@@ -105,6 +106,7 @@ $VERSION = '2.06';
     'airespace_apif'        => 'bsnAPIfOperStatus',
     'airespace_apif_oride'  => 'bsnAPIfWlanOverride',
     'airespace_apif_admin'  => 'bsnAPIfAdminStatus',
+    'airespace_apif_a_pwr'  => 'bsnAPIfAbsolutePowerList',
 
     # AIRESPACE-WIRELESS-MIB::bsnMobileStationTable
     'airespace_sta_mac'     => 'bsnMobileStationAPMacAddr',
@@ -146,6 +148,16 @@ $VERSION = '2.06';
     # AIRESPACE-SWITCHING-MIB::agentPortConfigTable
     'airespace_duplex_admin' => 'agentPortPhysicalMode',
     'airespace_duplex'       => 'agentPortPhysicalStatus',
+
+    # This needs to be cleaned up, but for now we pretend to
+    # have the CISCO-DOT11-MIB for signal strengths, etc.
+    'cd11_sigstrength' => 'bsnMobileStationRSSI', # kinda
+    'cd11_sigqual'     => 'bsnMobileStationSnr',  # kinda
+    'cd11_rxbyte'      => 'bsnMobileStationBytesReceived',
+    'cd11_txbyte'      => 'bsnMobileStationBytesSent',
+    'cd11_rxpkt'       => 'bsnMobileStationPacketsReceived',
+    'cd11_txpkt'       => 'bsnMobileStationPacketsSent',
+    'cd11_ssid'        => 'bsnMobileStationSsid',
 );
 
 %MUNGE = (
@@ -156,6 +168,8 @@ $VERSION = '2.06';
     'airespace_bl_mac'      => \&SNMP::Info::munge_mac,
     'airespace_if_mac'      => \&SNMP::Info::munge_mac,
     'airespace_sta_mac'     => \&SNMP::Info::munge_mac,
+    'cd11_rxpkt'            => \&munge_64bits,
+    'cd11_txpkt'            => \&munge_64bits,
 );
 
 sub layers {
@@ -419,11 +433,12 @@ sub i_mac {
             next unless defined $mac;
             $i_mac{$iid} = $mac;
         }
-
-        # Don't grab AP MACs - we want the AP to show up on edge switch
-        # ports
+        elsif ( $index =~ /(?:[0-9A-Fa-f]{2}:){5}[0-9A-Fa-f]{2}/ ) {
+            $index =~ s/\.\d+$//;
+            next unless defined $index;
+            $i_mac{$iid} = $index;
+        }
         # Some switch interfaces have MACs, virtuals report 00:00:00:00:00:00
-
         else {
             my $mac = $if_mac->{$iid};
             next unless defined $mac;
@@ -562,6 +577,13 @@ sub bp_index {
     return \%bp_index;
 }
 
+sub fw_mac {
+    my $airespace = shift;
+    my $partial   = shift;
+
+    return $airespace->SUPER::fw_mac($partial);
+}
+
 sub fw_port {
     my $airespace = shift;
     my $partial   = shift;
@@ -631,20 +653,96 @@ sub i_ssidbcast {
     my $airespace = shift;
     my $partial   = shift;
 
-    my $ssidlist = $airespace->i_ssidlist($partial)   || {};
-    my $bc_mode  = $airespace->airespace_bssid_mode() || 'enable';
+    my $ssidlist    = $airespace->i_ssidlist($partial)   || {};
+    my $bc_mode     = $airespace->airespace_bssid_mode() || 'enable';
+    my $ess_bc_mode = $airespace->airespace_ess_bcast()  || {};
+    my $ssids       = $airespace->airespace_ess_ssid()   || {};
+    my %ssid_index  = reverse %$ssids;
 
     my %bcmap     = qw/enable 1 disable 0/;
     my $broadcast = $bcmap{$bc_mode};
 
     my %i_ssidbcast;
     foreach my $iid ( keys %$ssidlist ) {
-        $i_ssidbcast{$iid} = $broadcast;
+        if (!$broadcast) {
+            $i_ssidbcast{$iid} = $broadcast;
+            next;
+        }
+        else {
+            my $ssid     = $ssidlist->{$iid};
+            my $ssid_idx = $ssid_index{$ssid};
+            my $bc       = $ess_bc_mode->{$ssid_idx};
+            $i_ssidbcast{$iid} = $bcmap{$bc};
+        }
     }
 
     return \%i_ssidbcast;
 }
 
+# Secret decoder ring for BSSID: https://supportforums.cisco.com/docs/DOC-2935
+# We need radio base MAC, SSID index, radio band, and whether AP is
+# VxWorks or IOS based then we can do the arithmetic
+
+sub i_ssidmac {
+    my $airespace = shift;
+    my $partial   = shift;
+
+    my $ssidlist  = $airespace->i_ssidlist($partial)  || {};
+    my $apif_type = $airespace->airespace_apif_type() || {};
+    my $ap_ios    = $airespace->bsnAPIOSVersion()     || {};
+
+    my %i_ssidmac;
+    foreach my $oid ( keys %$ssidlist ) {
+
+        my @parts    = split( /\./, $oid );
+        my $ssid_idx = pop(@parts);
+        my $slot     = pop(@parts);
+        my $last     = pop(@parts);
+
+        my $iid = $oid;
+
+        # Get radio band
+        $iid =~ s/\.\d+$//;
+        my $ap_type = $apif_type->{$iid};
+
+        # Determine if IOS based
+        $iid =~ s/\.\d+$//;
+        my $ios = $ap_ios->{$iid} || '';
+
+        # 17 can be used as index for 3rd Party AP's.  We only have one
+        # hexadecimal digit to work with so skip if outside the range
+        if ( $ssid_idx > 0 and $ssid_idx < 17 ) {
+
+            # Four cases:
+            # IOS and 2.4Ghz count up, starts at zero
+            if ( $ios and $ap_type =~ /b$/ ) {
+                $last = $last + ( $ssid_idx - 1 );
+            }
+
+            # IOS and 5Ghz - count down from maximum of 16
+            elsif ( $ios and $ap_type =~ /a$/ ) {
+                $last = $last + ( 16 - $ssid_idx );
+            }
+
+            # VxWorks and 5Ghz - count up, starts at zero
+            elsif ( $ios and $ap_type =~ /a$/ ) {
+                $last = $last + ( $ssid_idx - 1 );
+            }
+
+            # VxWorks and 2.4Ghz - count down from maximum of 16
+            else {
+                $last = $last + ( 16 - $ssid_idx );
+            }
+        }
+
+        push( @parts, $last );
+        my $bssid = join( ':', map { sprintf( "%02x", $_ ) } @parts );
+        $i_ssidmac{$oid} = $bssid;
+    }
+
+    return \%i_ssidmac;
+}
+
 sub i_80211channel {
     my $airespace = shift;
     my $partial   = shift;
@@ -661,6 +759,28 @@ sub i_80211channel {
     return \%i_80211channel;
 }
 
+sub dot11_cur_tx_pwr_mw {
+    my $airespace = shift;
+    my $partial   = shift;
+    my $cur       = $airespace->airespace_apif_power($partial);
+    my $pwr_abs   = $airespace->airespace_apif_a_pwr($partial);
+    
+    my $dot11_cur_tx_pwr_mw = {};
+    foreach my $idx ( keys %$cur ) {
+        my $pwr = $cur->{$idx};
+        if ( $pwr >= 1 && $pwr <= 8 ) {
+
+            my @pwr_list = split(/,/, $pwr_abs->{$idx} ); 
+            $dot11_cur_tx_pwr_mw->{$idx} = $pwr_list[$pwr-1]; 
+
+        }
+        else {
+            next;
+        }
+    }
+    return $dot11_cur_tx_pwr_mw;
+}
+
 # Pseudo ENTITY-MIB methods
 
 sub e_index {
@@ -940,6 +1060,13 @@ sub at_netaddr {
     return $ret;
 }
 
+sub munge_64bits {
+    # The controller sometimes hands off a ridiculous value for packets.
+    # Just truncate it to 32 bits.
+    my $value = shift;
+    return $value & 0xffffffff;
+}
+
 1;
 __END__
 
@@ -989,7 +1116,7 @@ Do not use directly.
 
 =over
 
-None.
+=item None.
 
 =back
 
@@ -1115,12 +1242,9 @@ switch.
 
 =item $airespace->layers()
 
-Returns 00000011.  Class emulates Layer 2 functionality for Thin APs through
+Returns 00000111.  Class emulates Layer 2 functionality for Thin APs through
 proprietary MIBs.
 
-=item $airespace->at_netaddr()
-=item $airespace->at_paddr()
-
 =item $airespace->serial()
 
 (C<agentInventorySerialNumber>)
@@ -1147,6 +1271,16 @@ Returns reference to hash.  Indicates whether the SSID is broadcast.
 Returns reference to hash.  Current operating frequency channel of the radio
 interface.
 
+=item $airespace->dot11_cur_tx_pwr_mw()
+
+Returns reference to hash.  Current transmit power, in milliwatts, of the
+radio interface.
+
+=item $airespace->i_ssidmac()
+
+With the same keys as i_ssidlist, returns the Basic service set
+identification (BSSID), MAC address, the AP is using for the SSID. 
+
 =back
 
 =head2 Dot11 Ess Table  (C<bsnDot11EssTable>)
@@ -1214,6 +1348,12 @@ enabled.
 
 (C<bsnDot11EssAclName>)           
 
+=item $airespace->airespace_ess_bcast()
+
+This attribute when enabled allows the switch to broadcast this SSID.
+
+(C<bsnDot11EssBroadcastSsid>)
+
 =back
 
 =head2 AP Table (C<bsnAPTable>)
@@ -1317,6 +1457,12 @@ However, if this is enabled, then only those WLANs that appear in the
 
 (C<bsnAPIfAdminStatus>)
 
+=item $airespace->airespace_apif_a_pwr()
+
+List of comma separated absolute power levels supported by the radio.
+
+(C<bsnAPIfAbsolutePowerList>)
+
 =back
 
 =head2 Mobile Station Table (C<bsnMobileStationTable>)
@@ -1546,15 +1692,15 @@ airespace_if_name() for virtual interfaces.
 
 =item $airespace->i_description()
 
-Returns reference to map of IIDs to interface types.  Returns C<ifDescr>
-for Ethernet interfaces, airespace_ap_loc() for thin AP interfaces, and
-airespace_if_name() for virtual interfaces.
+Returns reference to map of IIDs to interface descriptions.  Returns
+C<ifDescr> for Ethernet interfaces, airespace_ap_loc() for thin AP
+interfaces, and airespace_if_name() for virtual interfaces.
 
 =item $airespace->i_type()
 
-Returns reference to map of IIDs to interface descriptions.  Returns
-C<ifType> for Ethernet interfaces, airespace_apif_type() for thin AP
-interfaces, and airespace_if_type() for virtual interfaces.
+Returns reference to map of IIDs to interface types.  Returns C<ifType>
+for Ethernet interfaces, airespace_apif_type() for thin AP interfaces, and
+airespace_if_type() for virtual interfaces.
 
 =item $airespace->i_up()
 
@@ -1677,4 +1823,32 @@ entity which 'contains' this entity.
 
 =back
 
+=head2 Arp Cache Table Augmentation
+
+The controller has knowledge of MAC->IP mappings for wireless clients.
+Augmenting the arp cache data with these MAC->IP mappings enables visibility
+for stations that only communicate locally.
+
+=over
+
+=item $airespace->at_paddr()
+
+Adds MAC addresses extracted from the index of C<bsnMobileStationIpAddress>.
+
+=item $airespace->at_netaddr()
+
+Adds IP addresses from C<bsnMobileStationIpAddress>.
+
+=back
+
+=head1 Data Munging Callback Subroutines
+
+=over
+
+=item munge_64bits()
+
+Truncate packet values to 32 bits.
+
+=back
+
 =cut
