@@ -41,7 +41,7 @@ use SNMP::Info;
 
 use vars qw/$VERSION %MIBS %FUNCS %GLOBALS %MUNGE/;
 
-$VERSION = '2.11';
+$VERSION = '3.11';
 
 %MIBS = (
     'CISCO-VTP-MIB'                       => 'vtpVlanName',
@@ -191,6 +191,22 @@ sub i_vlan {
     return \%i_vlans;
 }
 
+sub i_untagged {
+    my $vtp = shift;
+    my ( $ifindex ) = @_;
+
+    # cannot defer to i_vlan - vtp_trunk_dyn_stat is not useful for down ports
+    # so we use vtp_trunk_dyn as a hint to use i_pvid
+
+    my $trunking = eval { $vtp->vtp_trunk_dyn($ifindex)->{$ifindex} };
+    if ($trunking and (($trunking eq 'on') or ($trunking eq 'onNoNegotiate'))) {
+        return $vtp->i_pvid(@_);
+    }
+    else {
+        return $vtp->i_vlan(@_);
+    }
+}
+
 sub i_vlan_membership {
     my $vtp     = shift;
     my $partial = shift;
@@ -306,6 +322,22 @@ sub set_i_vlan {
     return;
 }
 
+sub set_i_untagged {
+    my $vtp = shift;
+    my ( $vlan_id, $ifindex ) = @_;
+
+    # cannot defer to i_vlan - vtp_trunk_dyn_stat is not useful for down ports
+    # so we use vtp_trunk_dyn as a hint to use i_pvid
+
+    my $trunking = eval { $vtp->vtp_trunk_dyn($ifindex)->{$ifindex} };
+    if ($trunking and (($trunking eq 'on') or ($trunking eq 'onNoNegotiate'))) {
+        return $vtp->set_i_pvid(@_);
+    }
+    else {
+        return $vtp->set_i_vlan(@_);
+    }
+}
+
 sub set_add_i_vlan_tagged {
     my $vtp = shift;
     my ( $vlan_id, $ifindex ) = @_;
@@ -540,6 +572,10 @@ Your device will only implement a subset of these methods.
 Returns a mapping between C<ifIndex> and assigned VLAN ID for access ports
 and the default VLAN ID for trunk ports.
 
+=item $vtp->i_untagged()
+
+An alias for C<i_vlan>.
+
 =item $vtp->i_vlan_membership()
 
 Returns reference to hash of arrays: key = C<ifIndex>, value = array of VLAN
@@ -835,6 +871,12 @@ port C<ifIndex>.  This method should only be used on trunk ports.
   $vtp->set_i_pvid('2', $if_map{'FastEthernet0/1'}) 
     or die "Couldn't change port default VLAN. ",$vtp->error(1);
 
+=item $vtp->set_i_untagged ( vlan, ifIndex )
+
+This method attempts to work out whether the port referenced by ifIndex is
+trunking, and if so will return the value of C<set_i_pvid>. Otherwise, the
+value of C<set_i_vlan> is returned.
+
 =item $vtp->set_add_i_vlan_tagged ( vlan, ifIndex )
 
 Adds the VLAN to the enabled VLANs list of the port, must be supplied with the
