@@ -39,7 +39,7 @@ use SNMP::Info;
 
 use vars qw/$VERSION %FUNCS %GLOBALS %MIBS %MUNGE/;
 
-$VERSION = '2.06';
+$VERSION = '3.11';
 
 %MIBS = (
     'LLDP-MIB'          => 'lldpLocSysCapEnabled',
@@ -67,6 +67,7 @@ $VERSION = '2.06';
     'lldp_rem_sysname'  => 'lldpRemSysName',
     'lldp_rem_sysdesc'  => 'lldpRemSysDesc',
     'lldp_rem_sys_cap'  => 'lldpRemSysCapEnabled',
+    'lldp_rem_cap_spt'  => 'lldpRemSysCapSupported',
 
     # LLDP-MIB::lldpRemManAddrTable
     'lldp_rman_addr' => 'lldpRemManAddrIfSubtype',
@@ -80,6 +81,7 @@ $VERSION = '2.06';
     'lldp_rem_port_desc' => \&SNMP::Info::munge_null,
     'lldp_sys_cap'       => \&SNMP::Info::munge_bits,
     'lldp_rem_sys_cap'   => \&SNMP::Info::munge_bits,
+    'lldp_rem_cap_spt'   => \&SNMP::Info::munge_bits,
 );
 
 sub hasLLDP {
@@ -97,12 +99,30 @@ sub lldp_if {
     my $lldp    = shift;
     my $partial = shift;
 
-    my $addr = $lldp->lldp_rem_pid($partial) || {};
-
+    my $addr    = $lldp->lldp_rem_pid($partial) || {};
+    my $i_descr = $lldp->i_description() || {};
+    my $i_alias = $lldp->i_alias() || {};
+    my %r_i_descr = reverse %$i_descr;
+    my %r_i_alias = reverse %$i_alias;
+    
     my %lldp_if;
     foreach my $key ( keys %$addr ) {
         my @aOID = split( '\.', $key );
         my $port = $aOID[1];
+        next unless $port;
+        # Local LLDP port may not equate to ifIndex, see LldpPortNumber TEXTUAL-CONVENTION in LLDP-MIB.
+        # Cross reference lldpLocPortDesc with ifDescr and ifAlias to get ifIndex,
+        # prefer ifDescr over ifAlias because using cross ref with description is correct behavior 
+        # according to the LLDP-MIB. Some devices (eg H3C gear) seem to use ifAlias though.
+        my $lldp_desc = $lldp->lldpLocPortDesc($port);
+        my $desc = $lldp_desc->{$port};
+        # If cross reference is successful use it, otherwise stick with lldpRemLocalPortNum
+        if ( exists $r_i_descr{$desc} ) {
+            $port = $r_i_descr{$desc};
+        } elsif ( exists $r_i_alias{$desc} ) {
+            $port = $r_i_alias{$desc};
+        }
+        
         $lldp_if{$key} = $port;
     }
     return \%lldp_if;
@@ -150,10 +170,15 @@ sub lldp_port {
     my %lldp_port;
     foreach my $key ( sort keys %$pid ) {
         my $port = $pdesc->{$key};
+        my $type = $ptype->{$key};
+        if ( $type and $type eq 'interfaceName' ) {
+            # If the pid claims to be an interface name,
+            # believe it.
+            $port = $pid->{$key};
+        }
         unless ($port) {
             $port = $pid->{$key};
             next unless $port;
-            my $type = $ptype->{$key};
             next unless $type;
 
           # May need to format other types in the future, i.e. Network address
@@ -163,7 +188,7 @@ sub lldp_port {
             }
         }
 
-        # Nortel lldpRemPortDesc doesn't match ifDescr, but we can still
+        # Avaya/Nortel lldpRemPortDesc doesn't match ifDescr, but we can still
         # figure out slot.port based upon lldpRemPortDesc
         if ( $port =~ /^(Unit\s+(\d+)\s+)?Port\s+(\d+)$/ ) {
             $port = defined $1 ? "$2.$3" : "$3";
@@ -191,18 +216,63 @@ sub lldp_id {
         # May need to format other types in the future
         if ( $type =~ /mac/ ) {
             $id = join( ':', map { sprintf "%02x", $_ } unpack( 'C*', $id ) );
-        }elsif ($type eq 'networkAddress'){
- 	    if ( length(unpack('H*', $id)) == 10 ){
- 		# IP address (first octet is sign, I guess)
- 		my @octets = (map { sprintf "%02x",$_ } unpack('C*', $id))[1..4];
- 		$id = join '.', map { hex($_) } @octets;
- 	    }
-	}
+        }
+        elsif ($type eq 'networkAddress') {
+            if ( length(unpack('H*', $id)) == 10 ) {
+                # IP address (first octet is sign, I guess)
+                my @octets = (map { sprintf "%02x",$_ } unpack('C*', $id))[1..4];
+                $id = join '.', map { hex($_) } @octets;
+            }
+        }
         $lldp_id{$key} = $id;
     }
     return \%lldp_id;
 }
 
+sub lldp_platform {
+    my $lldp    = shift;
+    my $partial = shift;
+
+    my $rid  = $lldp->lldp_rem_id($partial) || {};
+    my $desc = $lldp->lldp_rem_sysdesc($partial) || {};
+    my $name = $lldp->lldp_rem_sysname($partial) || {};
+    
+    my %lldp_platform;
+    foreach my $key (keys %$rid) {
+        $lldp_platform{$key} = $desc->{$key} || $name->{$key};
+    }
+    return \%lldp_platform;
+}
+
+sub lldp_cap {
+    my $lldp     = shift;
+    my $partial = shift;
+
+    my $lldp_caps  = $lldp->lldp_rem_cap_spt($partial)  || {};
+    
+    # Encoded as BITS which Perl Net-SNMP implementation doesn't seem to
+    # be able to enumerate for us, so we have to get it from the MIB
+    # and enumerate ourselves
+    my $oid = SNMP::translateObj('lldpRemSysCapEnabled',0,1) || '';
+    my $enums = ((ref {} eq ref $SNMP::MIB{$oid}{'enums'}) ? $SNMP::MIB{$oid}{'enums'} : {});
+    my %r_enums = reverse %$enums;
+
+    my %lldp_cap;
+    foreach my $key ( keys %$lldp_caps ) {
+        my $cap_bits = $lldp_caps->{$key};
+        next unless $cap_bits;
+        
+        my $count = 0;
+        foreach my $bit (split //,$cap_bits) {
+            if ( $bit ) {
+                push ( @{$lldp_cap{$key}}, $r_enums{$count});
+            }
+            $count++;
+        }
+    }
+    return \%lldp_cap;
+}
+
 #sub root_ip {
 #    my $lldp = shift;
 #
@@ -343,7 +413,7 @@ Nulls are removed before the value is returned.
 The string value used to identify the system description of the local system.
 If the local agent supports IETF RFC 3418, C<lldpLocSysDesc> object should
 have the same value of C<sysDesc> object.
- 
+
 Nulls are removed before the value is returned.
 
 (C<lldpLocSysDesc>)
@@ -391,14 +461,16 @@ to a hash.
 
 =item $lldp->lldp_id()
 
-Returns the string value used to identify the chassis component	associated
+Returns the string value used to identify the chassis component associated
 with the remote system.
 
 (C<lldpRemChassisId>)
 
 =item $lldp->lldp_if()
 
-Returns the mapping to the SNMP Interface Table.
+Returns the mapping to the SNMP Interface Table. Tries to cross reference 
+(C<lldpLocPortDesc>) with (C<ifDescr>) and (C<ifAlias>) to get (C<ifIndex>), 
+if unable defaults to (C<lldpRemLocalPortNum>).
 
 =item  $lldp->lldp_ip()
 
@@ -414,6 +486,17 @@ Currently only returns IPv4 or MAC addresses.
 
 Returns remote port ID
 
+=item $lldp->lldp_platform()
+
+Tries to return something useful from C<lldp_rem_sysdesc()> or
+C<lldp_rem_sysname()>.
+
+=item  $lldp->lldp_cap() 
+
+Returns hash of arrays with each array containing the system capabilities
+supported by the remote system.  Possible elements in the array are
+enumerated from C<LldpSystemCapabilitiesMap>.
+
 =back
 
 =head2 LLDP Remote Table (C<lldpRemTable>)
@@ -429,7 +512,7 @@ the remote system.
 
 =item $lldp->lldp_rem_id()
 
-Returns the string value used to identify the chassis component	associated
+Returns the string value used to identify the chassis component    associated
 with the remote system.
 
 (C<lldpRemChassisId>)
@@ -477,9 +560,9 @@ Nulls are removed before the value is returned.
 
 =item  $lldp->lldp_rem_sys_cap() 
 
-Returns which system capabilities are enabled on the local system.  Results
-are munged into an ascii binary string, LSB.  Each digit
-represents a bit from the table below:
+Returns which system capabilities are enabled on the remote system.  Results
+are munged into an ascii binary string, LSB.  Each digit represents a bit
+from the table below:
 
 =over
 
