@@ -1,5 +1,5 @@
 # SNMP::Info::Layer1::Bayhub
-# $Id: Bayhub.pm,v 1.21 2009/06/12 22:24:25 maxbaker Exp $
+# $Id$
 #
 # Copyright (c) 2008 Eric Miller, Max Baker
 # All rights reserved.
@@ -42,7 +42,7 @@ use SNMP::Info::Layer2;
 
 use vars qw/$VERSION %FUNCS %GLOBALS %MIBS %MUNGE/;
 
-$VERSION = '2.01';
+$VERSION = '3.11';
 
 %MIBS = (
     %SNMP::Info::Layer2::MIBS,
@@ -93,7 +93,7 @@ sub os {
 }
 
 sub vendor {
-    return 'nortel';
+    return 'avaya';
 }
 
 sub model {
@@ -334,6 +334,13 @@ sub bp_index {
     return \%bp_index;
 }
 
+sub fw_mac {
+    my $bayhub = shift;
+    my $partial   = shift;
+
+    return $bayhub->SUPER::fw_mac($partial);
+}
+
 sub fw_port {
     my $bayhub  = shift;
     my $partial = shift;
@@ -469,7 +476,7 @@ __END__
 
 =head1 NAME
 
-SNMP::Info::Layer1::Bayhub - SNMP Interface to Bay / Nortel Hubs
+SNMP::Info::Layer1::Bayhub - SNMP Interface to Bay/Nortel/Avaya Hubs
 
 =head1 AUTHOR
 
@@ -541,7 +548,7 @@ These are methods that return scalar value from SNMP
 
 =item $bayhub->vendor()
 
-Returns 'nortel'
+Returns 'avaya'
 
 =item $bayhub->os()
 
