@@ -50,7 +50,7 @@ use SNMP::Info::Layer3;
 
 use vars qw/$VERSION %GLOBALS %MIBS %FUNCS %MUNGE/;
 
-$VERSION = '2.05';
+$VERSION = '3.11';
 
 %MIBS = (
     %SNMP::Info::Layer3::MIBS,
@@ -106,6 +106,12 @@ $VERSION = '2.05';
 *SNMP::Info::Layer3::C4000::i_speed_admin
     = \&SNMP::Info::MAU::mau_i_speed_admin;
 
+*SNMP::Info::Layer3::C4000::set_i_duplex_admin
+    = \&SNMP::Info::MAU::mau_set_i_duplex_admin;
+*SNMP::Info::Layer3::C4000::set_i_speed_admin
+    = \&SNMP::Info::MAU::mau_set_i_speed_admin;
+
+
 sub fan {
     my $c4000     = shift;
     my $fan_state = $c4000->fan_state();
@@ -281,6 +287,18 @@ Returns either (auto,full,half).
 
 Returns administrative speed for interfaces.
 
+=item $c4000->set_i_speed_admin(speed, ifIndex)
+
+Sets port speed, must be supplied with speed and port C<ifIndex>.
+
+Speed choices are '10', '100', '1000', 'auto'.
+
+=item $c4000->set_i_duplex_admin(duplex, ifIndex)
+
+Sets port duplex, must be supplied with duplex and port C<ifIndex>.
+
+Duplex choices are 'auto', 'half', 'full'.
+
 =back
 
 =head2 Table Methods imported from SNMP::Info::CiscoVTP
